@@ -1,4 +1,3 @@
-import tinychain.app
 import tinychain.error
 import tinychain.graph
 import tinychain.host
@@ -8,6 +7,7 @@ import tinychain.math.linalg
 import tinychain.ml
 import tinychain.ml.service
 import tinychain.scalar
+import tinychain.service
 
 from tinychain.collection import Column, btree, table, tensor
 from tinychain.context import Context
@@ -50,7 +50,7 @@ from tinychain.scalar.ref import (
     same_as,
     switch_case,
     while_loop,
-)  # TODO: replace with helper methods
-from tinychain.scalar.value import EmailAddress, Id, String, Value
+)
+from tinychain.scalar.value import EmailAddress, Id, String, Value, Version
 from tinychain.state import Class, Instance, State, Stream
 from tinychain.uri import URI
