@@ -1,8 +1,8 @@
 """Generic error types."""
 
+from .json import to_json
 from .scalar.value import String
-from .uri import uri, URI
-from .context import deanonymize, to_json
+from .uri import URI
 
 
 class TinyChainError(Exception):
@@ -18,10 +18,15 @@ class TinyChainError(Exception):
         self.message = String(message).render(params) if params else message
 
     def __json__(self):
-        return {str(uri(self)): [to_json(self.message)]}
+        return {str(URI(self)): [to_json(self.message)]}
 
     def __ns__(self, cxt, name_hint):
-        deanonymize(self.message, cxt, name_hint + "_message")
+        from .scalar.ref import is_op_ref
+
+        cxt.deanonymize(self.message, name_hint + "_message")
+
+        if is_op_ref(self.message):
+            cxt.assign(self.message, name_hint + "_message")
 
 
 class BadRequest(TinyChainError):
@@ -29,7 +34,7 @@ class BadRequest(TinyChainError):
 
     CODE = 400
 
-    __uri__ = uri(TinyChainError) + "/bad_request"
+    __uri__ = URI(TinyChainError) + "/bad_request"
 
 
 class Conflict(TinyChainError):
@@ -37,7 +42,7 @@ class Conflict(TinyChainError):
 
     CODE = 409
 
-    __uri__ = uri(TinyChainError) + "/conflict"
+    __uri__ = URI(TinyChainError) + "/conflict"
 
 
 class Forbidden(TinyChainError):
@@ -45,7 +50,7 @@ class Forbidden(TinyChainError):
 
     CODE = 403
 
-    __uri__ = uri(TinyChainError) + "/forbidden"
+    __uri__ = URI(TinyChainError) + "/forbidden"
 
 
 class MethodNotAllowed(TinyChainError):
@@ -56,7 +61,7 @@ class MethodNotAllowed(TinyChainError):
 
     CODE = 405
 
-    __uri__ = uri(TinyChainError) + "/method_not_allowed"
+    __uri__ = URI(TinyChainError) + "/method_not_allowed"
 
 
 class NotFound(TinyChainError):
@@ -64,7 +69,7 @@ class NotFound(TinyChainError):
 
     CODE = 404
 
-    __uri__ = uri(TinyChainError) + "/not_found"
+    __uri__ = URI(TinyChainError) + "/not_found"
 
 
 class NotImplemented(TinyChainError):
@@ -74,7 +79,7 @@ class NotImplemented(TinyChainError):
 
     CODE = 501
 
-    __uri__ = uri(TinyChainError) + "/not_implemented"
+    __uri__ = URI(TinyChainError) + "/not_implemented"
 
 
 class Timeout(TinyChainError):
@@ -82,7 +87,7 @@ class Timeout(TinyChainError):
 
     CODE = 408
 
-    __uri__ = uri(TinyChainError) + "/timeout"
+    __uri__ = URI(TinyChainError) + "/timeout"
 
 
 class Unauthorized(TinyChainError):
@@ -90,7 +95,7 @@ class Unauthorized(TinyChainError):
 
     CODE = 401
 
-    __uri__ = uri(TinyChainError) + "/unauthorized"
+    __uri__ = URI(TinyChainError) + "/unauthorized"
 
 
 class UnknownError(TinyChainError):
@@ -98,4 +103,4 @@ class UnknownError(TinyChainError):
 
     CODE = 500
 
-    __uri__ = uri(TinyChainError) + "/unknown"
+    __uri__ = URI(TinyChainError) + "/unknown"
