@@ -1,7 +1,7 @@
 from ..interface import Equality
+from ..json import to_json
 from ..state import State
-from ..uri import uri
-from ..context import to_json
+from ..uri import URI
 
 
 class Scalar(State, Equality):
@@ -11,10 +11,10 @@ class Scalar(State, Equality):
     Do not subclass :class:`Scalar` directly. Use :class:`Value` instead.
     """
 
-    __uri__ = uri(State) + "/scalar"
+    __uri__ = URI(State) + "/scalar"
 
     def __json__(self):
-        from .ref.helpers import form_of
+        from .ref.functions import form_of
         return to_json(form_of(self))
 
     def eq(self, other):
