@@ -1,6 +1,6 @@
+from ..json import to_json
 from ..state import State
 from ..uri import URI
-from ..context import to_json
 
 from .ref import form_of, Ref
 
